@@ -2,6 +2,7 @@ import pytest
 import sys
 from rpy2 import robjects
 from rpy2 import rinterface
+import rpy2.rlike.container
 import rpy2.robjects.conversion as conversion
 r = robjects.r
 
@@ -20,46 +21,13 @@ except:
     numpy = DummyNamespace()
 
 
-@pytest.fixture()
-def numpy_conversion():
-    rpyn.activate()
-    yield
-    rpyn.deactivate()
-
-
 @pytest.mark.skipif(not has_numpy,
                     reason='package numpy cannot be imported')
-@pytest.mark.usefixtures('numpy_conversion')
 class TestNumpyConversions(object):
 
-    def test_activate(self):
-        rpyn.deactivate()
-        #FIXME: is the following still making sense ?
-        assert rpyn.py2rpy != conversion.py2rpy
-        l = len(conversion.py2rpy.registry)
-        k = set(conversion.py2rpy.registry.keys())
-        rpyn.activate()
-        assert len(conversion.py2rpy.registry) > l
-        rpyn.deactivate()
-        assert len(conversion.py2rpy.registry) == l
-        assert set(conversion.py2rpy.registry.keys()) == k
-
-    def test_activate_twice(self):
-        rpyn.deactivate()
-        #FIXME: is the following still making sense ?
-        assert rpyn.py2rpy != conversion.py2rpy
-        l = len(conversion.py2rpy.registry)
-        k = set(conversion.py2rpy.registry.keys())
-        rpyn.activate()
-        rpyn.deactivate()
-        rpyn.activate()
-        assert len(conversion.py2rpy.registry) > l
-        rpyn.deactivate()
-        assert len(conversion.py2rpy.registry) == l
-        assert set(conversion.py2rpy.registry.keys()) == k
-
     def check_homogeneous(self, obj, mode, storage_mode):
-        converted = conversion.py2rpy(obj)
+        with (robjects.default_converter + rpyn.converter).context() as cv:
+            converted = cv.py2rpy(obj)
         assert r["mode"](converted)[0] == mode
         assert r["storage.mode"](converted)[0] == storage_mode
         assert list(obj) == list(converted)
@@ -77,7 +45,7 @@ class TestNumpyConversions(object):
         i = numpy.array(l, dtype="i")
         i_r = self.check_homogeneous(i, "numeric", "integer")
         assert tuple(l) == tuple(i_r)
-        
+
     def test_vector_float(self):
         l = [1.0, 2.0, 3.0]
         f = numpy.array(l, dtype="f")
@@ -102,15 +70,16 @@ class TestNumpyConversions(object):
     def test_vector_bytes(self):
         l = [b'a', b'b', b'c']
         s = numpy.array(l, dtype = '|S1')
-        converted = conversion.py2rpy(s)
+        with (robjects.default_converter + rpyn.converter).context() as cv:
+            converted = cv.py2rpy(s)
         assert r["mode"](converted)[0] == 'raw'
         assert r["storage.mode"](converted)[0] == 'raw'
         assert bytearray(b''.join(l)) == bytearray(converted)
 
     def test_array(self):
-
         i2d = numpy.array([[1, 2, 3], [4, 5, 6]], dtype='i')
-        i2d_r = conversion.py2rpy(i2d)
+        with (robjects.default_converter + rpyn.converter).context() as cv:
+            i2d_r = cv.py2rpy(i2d)
         assert r['storage.mode'](i2d_r)[0] == 'integer'
         assert tuple(r['dim'](i2d_r)) == (2, 3)
 
@@ -118,7 +87,8 @@ class TestNumpyConversions(object):
         assert r['['](i2d_r, 1, 2)[0] == i2d[0, 1]
 
         f3d = numpy.arange(24, dtype='f').reshape((2, 3, 4))
-        f3d_r = conversion.py2rpy(f3d)
+        with (robjects.default_converter + rpyn.converter).context() as cv:
+            f3d_r = cv.py2rpy(f3d)
 
         assert r['storage.mode'](f3d_r)[0] == 'double'
         assert tuple(r['dim'](f3d_r)) == (2, 3, 4)
@@ -135,7 +105,8 @@ class TestNumpyConversions(object):
     )
     def test_scalar_int(self, constructor):
         np_value = constructor(100)
-        r_vec = conversion.py2rpy(np_value)
+        with (robjects.default_converter + rpyn.converter).context() as cv:
+            r_vec = cv.py2rpy(np_value)
         r_scalar = numpy.array(r_vec)[0]
         assert np_value == r_scalar
 
@@ -143,13 +114,15 @@ class TestNumpyConversions(object):
                         reason='numpy.float128 not available on this system')
     def test_scalar_f128(self):
         f128 = numpy.float128(100.000000003)
-        f128_r = conversion.py2rpy(f128)
+        with (robjects.default_converter + rpyn.converter).context() as cv:
+            f128_r = cv.py2rpy(f128)
         f128_test = numpy.array(f128_r)[0]
         assert f128 == f128_test
 
     def test_object_array(self):
         o = numpy.array([1, "a", 3.2], dtype=numpy.object_)
-        o_r = conversion.py2rpy(o)
+        with (robjects.default_converter + rpyn.converter).context() as cv:
+            o_r = cv.py2rpy(o)
         assert r['mode'](o_r)[0] == 'list'
         assert r['[['](o_r, 1)[0] == 1
         assert r['[['](o_r, 2)[0] == 'a'
@@ -158,7 +131,8 @@ class TestNumpyConversions(object):
     def test_record_array(self):
         rec = numpy.array([(1, 2.3), (2, -0.7), (3, 12.1)],
                           dtype=[("count", "i"), ("value", numpy.double)])
-        rec_r = conversion.py2rpy(rec)
+        with (robjects.default_converter + rpyn.converter).context() as cv:
+            rec_r = cv.py2rpy(rec)
         assert r["is.data.frame"](rec_r)[0] is True
         assert tuple(r["names"](rec_r)) == ("count", "value")
         count_r = rec_r[rec_r.names.index('count')]
@@ -171,34 +145,61 @@ class TestNumpyConversions(object):
     def test_bad_array(self):
         u = numpy.array([1, 2, 3], dtype=numpy.uint32)
         with pytest.raises(ValueError):
-            conversion.py2rpy(u)
+            with (robjects.default_converter + rpyn.converter).context() as cv:
+                cv.py2rpy(u)
 
     def test_assign_numpy_object(self):
         x = numpy.arange(-10., 10., 1)
         env = robjects.Environment()
-        env['x'] = x
+        with (robjects.default_converter + rpyn.converter).context() as cv:
+            env['x'] = x
         assert len(env) == 1
         # do have an R object of the right type ?
-        x_r = env['x']
+        with robjects.default_converter.context() as lc:
+            x_r = env['x']
+
         assert robjects.rinterface.RTYPES.REALSXP == x_r.typeof
         #
         assert tuple(x_r.dim) == (20,)
 
-
     def test_dataframe_to_numpy(self):
         df = robjects.vectors.DataFrame(
             {'a': 1,
              'b': 2,
-             'c': robjects.vectors.FactorVector('e')})
-        rec = conversion.rpy2py(df)
+             'c': robjects.vectors.FactorVector('e'),
+             'd': robjects.vectors.StrVector(['xyz'])})
+        with conversion.localconverter(robjects.default_converter + rpyn.converter) as cv:
+            rec = cv.rpy2py(df)
         assert numpy.recarray == type(rec)
         assert rec.a[0] == 1
         assert rec.b[0] == 2
         assert rec.c[0] == 'e'
+        assert rec.d[0] == 'xyz'
+
+    @pytest.mark.parametrize(
+        'cls',
+        (robjects.ListVector,
+         rinterface.ListSexpVector)
+    )
+    def test_rlist_to_numpy(self, cls):
+        df = cls(
+            robjects.ListVector(
+                {'a': 1,
+                 'b': 2,
+                 'c': robjects.vectors.FactorVector('e')}
+            )
+        )
+        with (robjects.default_converter + rpyn.converter).context() as cv:
+            rec = cv.rpy2py(df)
+        assert rpy2.rlike.container.OrdDict == type(rec)
+        assert rec['a'][0] == 1
+        assert rec['b'][0] == 2
+        assert rec['c'][0] == 1  # not 'e'.
 
     def test_atomic_vector_to_numpy(self):
         v = robjects.vectors.IntVector((1,2,3))
-        a = rpyn.rpy2py(v)
+        with rpyn.converter.context() as cv:
+            a = cv.rpy2py(v)
         assert isinstance(a, numpy.ndarray)
         assert v[0] == 1
 
@@ -206,10 +207,21 @@ class TestNumpyConversions(object):
         df = robjects.vectors.DataFrame({
             "A": robjects.vectors.IntVector([1,2,3]),
             "B": robjects.vectors.IntVector([1,2,3])})
-        b = df.rx2('B')
+        with (robjects.default_converter + rpyn.converter).context() as cv:
+            b = df.rx2('B')
         assert tuple((1,2,3)) == tuple(b)
 
-    
+    def test_rint_to_numpy(self):
+        with (robjects.default_converter + rpyn.converter).context() as cv:
+            a = robjects.r('c(1:4)')
+        assert isinstance(a, numpy.ndarray)
+
+    def test_rfloat_to_numpy(self):
+        with (robjects.default_converter + rpyn.converter).context() as cv:
+            a = robjects.r('c(1.0, 2.0, 3.0)')
+        assert isinstance(a, numpy.ndarray)
+
+
 @pytest.mark.skipif(not has_numpy,
                     reason='package numpy cannot be imported')
 @pytest.mark.parametrize('dtype',
@@ -242,4 +254,26 @@ def test_numpy_O_py2rpy(values, expected_cls):
     a = numpy.array(values, dtype='O')
     v = rpyn.numpy_O_py2rpy(a)
     assert isinstance(v, expected_cls)
+
     
+@pytest.mark.skipif(not has_numpy,
+                    reason='package numpy cannot be imported')
+@pytest.mark.parametrize(
+    'rcode,expected_values',
+    (('c(TRUE, FALSE)', (True, False, )),
+     ('c(1, 2, 3)', (1, 2, 3)),
+     ('c(1.0, 2.0, 3.0)', (1.0, 2.0, 3.0)),
+     ('c("ab", "cd", NA_character_)', ('ab', 'cd', None)))
+)
+def test_rpy2py(rcode, expected_values):
+    with (robjects.default_converter + rpyn.converter).context():
+        values = robjects.r(rcode)
+    assert tuple(values) == expected_values
+
+
+@pytest.mark.skipif(not has_numpy,
+                    reason='package numpy cannot be imported')
+def test_NA_CharSexp():
+    with (robjects.default_converter + rpyn.converter).context():
+        values = robjects.r('c(NA_character_)')
+    assert values[0] is None
